@@ -2,7 +2,7 @@
 // ip/detail/endpoint.hpp
 // ~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2022 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2025 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -28,48 +28,48 @@ namespace asio {
 namespace ip {
 namespace detail {
 
-// Helper class for implementating an IP endpoint.
+// Helper class for implementing an IP endpoint.
 class endpoint
 {
 public:
   // Default constructor.
-  ASIO_DECL endpoint() ASIO_NOEXCEPT;
+  ASIO_DECL endpoint() noexcept;
 
   // Construct an endpoint using a family and port number.
   ASIO_DECL endpoint(int family,
-      unsigned short port_num) ASIO_NOEXCEPT;
+      unsigned short port_num) noexcept;
 
   // Construct an endpoint using an address and port number.
   ASIO_DECL endpoint(const asio::ip::address& addr,
-      unsigned short port_num) ASIO_NOEXCEPT;
+      unsigned short port_num) noexcept;
 
   // Copy constructor.
-  endpoint(const endpoint& other) ASIO_NOEXCEPT
+  endpoint(const endpoint& other) noexcept
     : data_(other.data_)
   {
   }
 
   // Assign from another endpoint.
-  endpoint& operator=(const endpoint& other) ASIO_NOEXCEPT
+  endpoint& operator=(const endpoint& other) noexcept
   {
     data_ = other.data_;
     return *this;
   }
 
   // Get the underlying endpoint in the native type.
-  asio::detail::socket_addr_type* data() ASIO_NOEXCEPT
+  asio::detail::socket_addr_type* data() noexcept
   {
-    return &data_.base;
+    return &data_.base[0];
   }
 
   // Get the underlying endpoint in the native type.
-  const asio::detail::socket_addr_type* data() const ASIO_NOEXCEPT
+  const asio::detail::socket_addr_type* data() const noexcept
   {
-    return &data_.base;
+    return &data_.base[0];
   }
 
   // Get the underlying size of the endpoint in the native type.
-  std::size_t size() const ASIO_NOEXCEPT
+  std::size_t size() const noexcept
   {
     if (is_v4())
       return sizeof(asio::detail::sockaddr_in4_type);
@@ -81,36 +81,36 @@ public:
   ASIO_DECL void resize(std::size_t new_size);
 
   // Get the capacity of the endpoint in the native type.
-  std::size_t capacity() const ASIO_NOEXCEPT
+  std::size_t capacity() const noexcept
   {
     return sizeof(data_);
   }
 
   // Get the port associated with the endpoint.
-  ASIO_DECL unsigned short port() const ASIO_NOEXCEPT;
+  ASIO_DECL unsigned short port() const noexcept;
 
   // Set the port associated with the endpoint.
-  ASIO_DECL void port(unsigned short port_num) ASIO_NOEXCEPT;
+  ASIO_DECL void port(unsigned short port_num) noexcept;
 
   // Get the IP address associated with the endpoint.
-  ASIO_DECL asio::ip::address address() const ASIO_NOEXCEPT;
+  ASIO_DECL asio::ip::address address() const noexcept;
 
   // Set the IP address associated with the endpoint.
   ASIO_DECL void address(
-      const asio::ip::address& addr) ASIO_NOEXCEPT;
+      const asio::ip::address& addr) noexcept;
 
   // Compare two endpoints for equality.
   ASIO_DECL friend bool operator==(const endpoint& e1,
-      const endpoint& e2) ASIO_NOEXCEPT;
+      const endpoint& e2) noexcept;
 
   // Compare endpoints for ordering.
   ASIO_DECL friend bool operator<(const endpoint& e1,
-      const endpoint& e2) ASIO_NOEXCEPT;
+      const endpoint& e2) noexcept;
 
   // Determine whether the endpoint is IPv4.
-  bool is_v4() const ASIO_NOEXCEPT
+  bool is_v4() const noexcept
   {
-    return data_.base.sa_family == ASIO_OS_DEF(AF_INET);
+    return data_.base[0].sa_family == ASIO_OS_DEF(AF_INET);
   }
 
 #if !defined(ASIO_NO_IOSTREAM)
@@ -122,7 +122,11 @@ private:
   // The underlying IP socket address.
   union data_union
   {
-    asio::detail::socket_addr_type base;
+#if defined(_FORTIFY_SOURCE)
+    asio::detail::socket_addr_type base[8];
+#else // defined(_FORTIFY_SOURCE)
+    asio::detail::socket_addr_type base[1];
+#endif // defined(_FORTIFY_SOURCE)
     asio::detail::sockaddr_in4_type v4;
     asio::detail::sockaddr_in6_type v6;
   } data_;
