@@ -2,7 +2,7 @@
 // blocking_udp_echo_client.cpp
 // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2025 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -27,18 +27,19 @@ int main(int argc, char* argv[])
       return 1;
     }
 
-    asio::io_service io_service;
+    asio::io_context io_context;
 
-    udp::socket s(io_service, udp::endpoint(udp::v4(), 0));
+    udp::socket s(io_context, udp::endpoint(udp::v4(), 0));
 
-    udp::resolver resolver(io_service);
-    udp::endpoint endpoint = *resolver.resolve({udp::v4(), argv[1], argv[2]});
+    udp::resolver resolver(io_context);
+    udp::resolver::results_type endpoints =
+      resolver.resolve(udp::v4(), argv[1], argv[2]);
 
     std::cout << "Enter message: ";
     char request[max_length];
     std::cin.getline(request, max_length);
     size_t request_length = std::strlen(request);
-    s.send_to(asio::buffer(request, request_length), endpoint);
+    s.send_to(asio::buffer(request, request_length), *endpoints.begin());
 
     char reply[max_length];
     udp::endpoint sender_endpoint;
