@@ -2,7 +2,7 @@
 // prioritised_handlers.cpp
 // ~~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2022 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2025 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -142,13 +142,13 @@ private:
 
 //----------------------------------------------------------------------
 
-void high_priority_handler(const asio::error_code& /*ec*/,
+void high_priority_handler(const std::error_code& /*ec*/,
     tcp::socket /*socket*/)
 {
   std::cout << "High priority handler\n";
 }
 
-void middle_priority_handler(const asio::error_code& /*ec*/)
+void middle_priority_handler(const std::error_code& /*ec*/)
 {
   std::cout << "Middle priority handler\n";
 }
