@@ -2,13 +2,14 @@
 // composed_3.cpp
 // ~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2022 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2025 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
 
 #include <asio/bind_executor.hpp>
+#include <asio/deferred.hpp>
 #include <asio/io_context.hpp>
 #include <asio/ip/tcp.hpp>
 #include <asio/use_future.hpp>
@@ -37,12 +38,19 @@ template <typename CompletionToken>
 auto async_write_message(tcp::socket& socket,
     const char* message, CompletionToken&& token)
   // The return type of the initiating function is deduced from the combination
-  // of CompletionToken type and the completion handler's signature. When the
-  // completion token is a simple callback, the return type is always void.
-  // In this example, when the completion token is asio::yield_context
-  // (used for stackful coroutines) the return type would be also be void, as
+  // of:
+  //
+  // - the CompletionToken type,
+  // - the completion handler signature, and
+  // - the asynchronous operation's initiation function object.
+  //
+  // When the completion token is a simple callback, the return type is always
+  // void. In this example, when the completion token is asio::yield_context
+  // (used for stackful coroutines) the return type would also be void, as
   // there is no non-error argument to the completion handler. When the
-  // completion token is asio::use_future it would be std::future<void>.
+  // completion token is asio::use_future it would be std::future<void>. When
+  // the completion token is asio::deferred, the return type differs for each
+  // asynchronous operation.
   //
   // In C++14 we can omit the return type as it is automatically deduced from
   // the return type of asio::async_initiate.
@@ -149,6 +157,39 @@ void test_callback()
 
 //------------------------------------------------------------------------------
 
+void test_deferred()
+{
+  asio::io_context io_context;
+
+  tcp::acceptor acceptor(io_context, {tcp::v4(), 55555});
+  tcp::socket socket = acceptor.accept();
+
+  // Test our asynchronous operation using the deferred completion token. This
+  // token causes the operation's initiating function to package up the
+  // operation with its arguments to return a function object, which may then be
+  // used to launch the asynchronous operation.
+  auto op = async_write_message(socket,
+      "Testing deferred\r\n", asio::deferred);
+
+  // Launch the operation using a lambda as a callback.
+  std::move(op)(
+      [](const std::error_code& error)
+      {
+        if (!error)
+        {
+          std::cout << "Message sent\n";
+        }
+        else
+        {
+          std::cout << "Error: " << error.message() << "\n";
+        }
+      });
+
+  io_context.run();
+}
+
+//------------------------------------------------------------------------------
+
 void test_future()
 {
   asio::io_context io_context;
@@ -182,5 +223,6 @@ void test_future()
 int main()
 {
   test_callback();
+  test_deferred();
   test_future();
 }
