@@ -2,7 +2,7 @@
 // buffers_iterator.cpp
 // ~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2020 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2025 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -16,17 +16,10 @@
 // Test that header file is self-contained.
 #include "asio/buffers_iterator.hpp"
 
+#include <array>
 #include "asio/buffer.hpp"
 #include "unit_test.hpp"
 
-#if defined(ASIO_HAS_BOOST_ARRAY)
-# include <boost/array.hpp>
-#endif // defined(ASIO_HAS_BOOST_ARRAY)
-
-#if defined(ASIO_HAS_STD_ARRAY)
-# include <array>
-#endif // defined(ASIO_HAS_STD_ARRAY)
-
 //------------------------------------------------------------------------------
 
 // buffers_iterator_compile test
@@ -36,11 +29,7 @@
 
 namespace buffers_iterator_compile {
 
-#if defined(ASIO_HAS_BOOST_ARRAY)
-using boost::array;
-#elif defined(ASIO_HAS_STD_ARRAY)
 using std::array;
-#endif // defined(ASIO_HAS_BOOST_ARRAY)
 using std::vector;
 using namespace asio;
 
@@ -59,7 +48,6 @@ void test()
     vector<const_buffer> cb3;
     cb3.push_back(buffer(cdata1));
 
-
     // buffers_iterator constructors.
 
     buffers_iterator<mutable_buffer, char> bi1;
@@ -288,5 +276,5 @@ void test()
 ASIO_TEST_SUITE
 (
   "buffers_iterator",
-  ASIO_TEST_CASE(buffers_iterator_compile::test)
+  ASIO_COMPILE_TEST_CASE(buffers_iterator_compile::test)
 )
