@@ -2,7 +2,7 @@
 // datagram_protocol.cpp
 // ~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2020 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2025 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -74,7 +74,7 @@ void test()
     int native_socket2 = ::socket(AF_UNIX, SOCK_DGRAM, 0);
     dp::socket socket8(ioc_ex, dp(), native_socket2);
 
-    // basic_io_object functions.
+    // I/O object functions.
 
     dp::socket::executor_type ex = socket1.get_executor();
     (void)ex;
@@ -238,5 +238,5 @@ void test()
 ASIO_TEST_SUITE
 (
   "local/datagram_protocol",
-  ASIO_TEST_CASE(local_datagram_protocol_socket_compile::test)
+  ASIO_COMPILE_TEST_CASE(local_datagram_protocol_socket_compile::test)
 )
